package com.gic.demo.base.api.common;

import java.io.Serializable;

/**
 * 服务响应
 *
 * @param <T> 返回类型
 * @author zhurz
 */
public class ServiceResponse<T> implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * 响应代码（成功代码固定为 {@link Constant#SUCCESS 0000}）
     */
    private String code;

    /**
     * 响应信息（可为null，不一定和code说明信息一致）
     */
    private String message;

    /**
     * 响应结果
     */
    private T result;

    public static <T> ServiceResponse<T> success() {
        return new ServiceResponse<>(Constant.SUCCESS, null, null);
    }

    public static <T> ServiceResponse<T> success(T result) {
        return new ServiceResponse<>(Constant.SUCCESS, null, result);
    }

    public static <T> ServiceResponse<T> success(T result, String message) {
        return new ServiceResponse<>(Constant.SUCCESS, message, result);
    }

    public static <T> ServiceResponse<T> failure(String code, String message) {
        return new ServiceResponse<>(code, message, null);
    }

    public static <T> ServiceResponse<T> failure(String code, String message, T result) {
        return new ServiceResponse<>(code, message, result);
    }

    public ServiceResponse() {
    }

    private ServiceResponse(String code, String message, T result) {
        this.code = code;
        this.message = message;
        this.result = result;
    }

    /**
     * @return 是否成功
     */
    public boolean isSuccess() {
        return Constant.SUCCESS.equals(code);
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public T getResult() {
        return result;
    }

    public void setResult(T result) {
        this.result = result;
    }
}

